'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _xml2js = require('xml2js');

var _xml2js2 = _interopRequireDefault(_xml2js);

var _zlib = require('zlib');

var _zlib2 = _interopRequireDefault(_zlib);

var _rules = require('./rules');

var _rules2 = _interopRequireDefault(_rules);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var rulesArray = [];

_lodash2.default.forOwn(_rules2.default, function (value, ruleKey) {
  rulesArray.push({ ruleKey: ruleKey, func: _rules2.default[ruleKey] });
});

var lintResults = function lintResults(abletonObj) {
  return rulesArray.map(function (rule) {
    return rule.func(abletonObj);
  });
};

/* eslint-disable no-param-reassign */
var cleanPlainObjectNode = function cleanPlainObjectNode(node) {
  // Cleaning up [0] and $
  _lodash2.default.forOwn(node, function (el, key) {
    if (_lodash2.default.isArray(el) && el.length === 1) {
      if (_lodash2.default.keys(el[0]).length === 1 && _lodash2.default.has(el[0], '$')) {
        node[key] = el[0].$;
      } else {
        node[key] = el[0];
      }
    }
  });
  _lodash2.default.forOwn(node, function (el, key) {
    if (_lodash2.default.keys(el).length === 1 && _lodash2.default.has(el, '$')) {
      node[key] = el.$;
    }
  });
};

var cleanAbleton = function cleanAbleton(node) {
  if (_lodash2.default.isArray(node)) {
    node.forEach(function (el) {
      cleanAbleton(el);
    });
  } else if (_lodash2.default.isPlainObject(node)) {
    cleanPlainObjectNode(node);
    // Traversing and creating parenthood links (needs a 2nd forOwn
    // because entries have been affected by the first forOwn)
    _lodash2.default.forOwn(node, function (el) {
      cleanAbleton(el);
    });
  }
};

var linkDescendance = function linkDescendance(node, grandparentIfArray) {
  if (_lodash2.default.isArray(node)) {
    node.forEach(function (el) {
      linkDescendance(el, node);
    });
  } else if (_lodash2.default.isPlainObject(node)) {
    _lodash2.default.forOwn(node, function (el, key) {
      linkDescendance(el);
      if (_lodash2.default.isPlainObject(el) || _lodash2.default.isArray(el)) {
        el.codakitKey = key;
        el.codakitParent = node;
      }
    });
    node.codakitKey = 'parent-is-array';
    node.codakitParent = grandparentIfArray;
  }
};

var processAls = function processAls(path) {
  _fs2.default.readFile(path, function (err1, file) {
    _zlib2.default.gunzip(file, function (err2, gzip) {
      _xml2js2.default.parseString(gzip.toString(), function (err3, abletonObj) {
        cleanAbleton(abletonObj);
        linkDescendance(abletonObj);
        var results = lintResults(abletonObj.Ableton.LiveSet);
        var foundError = false;
        if (results.length) {
          results.forEach(function (typeOfIssue) {
            typeOfIssue.forEach(function (issue) {
              foundError = true;
              // eslint-disable-next-line no-console
              console.log(issue);
            });
          });
        }
        if (foundError) {
          process.exit(1);
        } else {
          // eslint-disable-next-line no-console
          console.log('🎉 All tests passed!');
        }
      });
    });
  });
};

exports.default = processAls;