'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

require('colors');

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _utils = require('../utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint-disable no-console */

var errors = [];

var testAutomationDots = function testAutomationDots(node, dots) {
  var numUselessDots = 0;
  if (dots.length === 2) {
    // One dot
    numUselessDots = 1;
  } else {
    dots.forEach(function (dot, i) {
      // Middle dots
      if (i > 0 && i < dots.length - 1) {
        if (dots[i] === dots[i - 1] && dots[i] === dots[i + 1]) {
          numUselessDots += 1;
        }
      }
    });
    if (dots[dots.length - 1] === dots[dots.length - 2]) {
      // Last dot
      numUselessDots += 1;
    }
  }
  if (numUselessDots > 0) {
    var paramName = node.codakitParent.codakitKey;
    var deviceAscendant = node.codakitParent;
    while (!(deviceAscendant.On && deviceAscendant.UserName && deviceAscendant.Annotation)) {
      deviceAscendant = deviceAscendant.codakitParent;
      if (!deviceAscendant) {
        // In case we're on a Send, we avoid crashing here
        return;
      }
    }

    var deviceName = deviceAscendant.codakitKey;
    var trackAscendant = deviceAscendant.codakitParent;
    while (!trackAscendant.TrackUnfolded) {
      trackAscendant = trackAscendant.codakitParent;
    }
    var trackName = trackAscendant.Name.EffectiveName.Value;
    if (!(0, _utils.isExceptionTrack)(trackAscendant)) {
      errors.push('There are ' + numUselessDots + ' useless automation dots on ' + trackName.green + '/' + deviceName.green + '/' + paramName.green);
    }
  }
};

function traverse(node) {
  if (node.codakitKey === 'ArrangerAutomation') {
    if (node.Events.FloatEvent && _lodash2.default.isArray(node.Events.FloatEvent)) {
      testAutomationDots(node, node.Events.FloatEvent.map(function (dot) {
        return dot.$.Value;
      }));
    }
    if (node.Events.BoolEvent && _lodash2.default.isArray(node.Events.BoolEvent)) {
      testAutomationDots(node, node.Events.BoolEvent.map(function (dot) {
        return dot.$.Value;
      }));
    }
  }
  if (_lodash2.default.isArray(node)) {
    node.forEach(function (el) {
      traverse(el);
    });
  } else if (_lodash2.default.isPlainObject(node)) {
    _lodash2.default.forOwn(node, function (el, key) {
      if (key !== 'codakitParent') {
        traverse(el);
      }
    });
  }
}

exports.default = function (abletonObj) {
  traverse(abletonObj);
  var tmpErrors = errors;
  errors = [];
  return tmpErrors;
};